#include "HistogramWin.h"
#include <string.h>
#include <sgl/transform3.h>
#include "jed.h"
#include <stdio.h>

HistogramWin::HistogramWin()
{
  // Nothing connected by default
  dataImage.pixels=NULL;

  // Default selection
  xselection=yselection=selectionsize=0;

  // Default window size
  glutSetWindow(glWindowNum);
  glutReshapeWindow(200,200);

  // Default sample the whole image
  maxPointsToSample=1000000;

  // Default Title
  strcpy(title,"Histogram");
}

HistogramWin::~HistogramWin()
{
}

void renderString (int x, int y, char *str)
{
  // We do bad things to the raster position
  // We should use glGet to get the old value in order to
  // properly reset it.
  int len=strlen(str);
  
  // position set
  glRasterPos2f(x,y);

  // Loop over each character
  for (int i=0;i<len;i++)
    {
      glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, str[i]);
      x=x+glutBitmapWidth(GLUT_BITMAP_HELVETICA_10, str[i]);
    }

  glRasterPos2f(0,200);
  glFinish();
}

void HistogramWin::displayTitle()
{
  glColor3f(.9,.9,.9);
  renderString(5,height-15,title);
}

void HistogramWin::display()
{
  // Make sure a valid dataImage exists
  if (dataImage.pixels==NULL)
    return;
 
  // Figure step size
  int step = selectionsize / sqrt(maxPointsToSample);
  if (step <1 ) step =1;

  // counting arrays
  int rhist[256], ghist[256], bhist[256];
  for (int i=0;i<257;++i)
    {
      rhist[i]=ghist[i]=bhist[i]=0;
    }

  //printf ("%d %d %d - %d %d \n",selectionsize,xselection,yselection,
  //  dataImage.xsize,dataImage.ysize);
  
  // Sample the image
  int numSamples=0;
  for (int x=xselection-selectionsize; x< xselection+selectionsize; x+=step)
    for (int y=yselection-selectionsize; y< yselection+selectionsize; y+=step)
      {
	//Vector3     rgb;
	int rval =dataImage.r(x,y);
	//	rgb.y=dataImage.g(x,y);
	//	rgb.z=dataImage.b(x,y);

	//cout << "histrgb " << rgb << "\n";

	//	printf ("rval [%d %d %d] ",x,y,rval);
	rhist[rval] +=1;
	ghist[dataImage.g(x,y)] +=1;
	bhist[dataImage.b(x,y)] +=1;
	numSamples++;
      }

  // Clear window
  glClearColor(0,0,0,1);
  glClear(GL_COLOR_BUFFER_BIT);

  // displayTitle
  displayTitle();
 
  double maxHistValue=4*numSamples*6.0/255.0;
 
  
  /*
  for (i=0;i<256;i++)
    {
      maxHistValue=MAX(maxHistValue,rhist[i]);
      maxHistValue=MAX(maxHistValue,ghist[i]);
      maxHistValue=MAX(maxHistValue,bhist[i]);
      //  printf ("%d %d %d\n",rhist[i],ghist[i],bhist[i]);
     
    }
  */
  double maxHistMult = (double)height/(double)maxHistValue;
  //printf ("%d %d %f\n",height,maxHistValue,maxHistMult);
  //printf ("%d %f %f\n",selectionsize,maxHistValue,maxHistMult);
  //int mr=0;
  //int mrh=0;
  for (int c=0; c<255; c++)
    {
      
      int i=c*width/255;
      int id=(c+1)*width/255;
      int x=c;
      int xd=c+1;
      int ry=((double)(rhist[x]))*maxHistMult;
      int ryd=((double)(rhist[xd]))*maxHistMult;
      //mr=MAX(mr,ry);
      //mrh=MAX(mrh,rhist[x]);
      // RED
      glColor3f(1,0,0);
      glBegin(GL_LINES);
      glVertex2s(i  ,ry);
      glVertex2s(id, ryd);
      glEnd();
      // GREEN
      glColor3f(0,1,0);
      glBegin(GL_LINES);
      glVertex2s(i  ,(double)ghist[x]*maxHistMult);
      glVertex2s(id,(double)ghist[xd]*maxHistMult);
      glEnd();
      // BLUE
      glColor3f(0,0,1);
      glBegin(GL_LINES);
      glVertex2s(i  ,(double)bhist[x]*maxHistMult);
      glVertex2s(id,(double)bhist[xd]*maxHistMult);
      glEnd();

      
    }
  //printf ("R %d %d\n",mr,mrh);

}


